@@ -1,4 +1,5 @@
 import json
+import operator
 import re
 from distutils.version import LooseVersion
 
@@ -14,6 +15,7 @@ from django.urls import re_path, reverse
 from django.utils.decorators import method_decorator
 from django.utils.encoding import force_str
 from django.utils.translation import gettext
+from django.utils.translation.trans_real import get_language
 from django.views.decorators.clickjacking import xframe_options_sameorigin
 from django.views.decorators.http import require_POST
 
@@ -21,7 +23,7 @@ import cms
 from cms.models import CMSPlugin
 from cms.plugin_base import CMSPluginBase
 from cms.plugin_pool import plugin_pool
-from cms.utils.placeholder import get_toolbar_plugin_struct
+from cms.utils.placeholder import get_placeholder_conf
 from cms.utils.urlutils import admin_reverse
 
 from . import settings
@@ -34,7 +36,7 @@ from .utils import (
 from .widgets import TextEditorWidget
 
 
-CMS_34 = LooseVersion(cms.__version__) >= LooseVersion('3.4')
+CMS_34 = LooseVersion(cms.__version__) >= LooseVersion("3.4")
 
 
 def _user_can_change_placeholder(request, placeholder):
@@ -48,7 +50,7 @@ def post_add_plugin(operation, **kwargs):
     from djangocms_history.helpers import get_bound_plugins, get_plugin_data
     from djangocms_history.models import dump_json
 
-    text_plugin = kwargs['plugin']
+    text_plugin = kwargs["plugin"]
     new_plugin_ids = set(text_plugin._get_inline_plugin_ids())
 
     if not new_plugin_ids:
@@ -59,21 +61,21 @@ def post_add_plugin(operation, **kwargs):
     new_plugins = get_bound_plugins(new_plugins)
 
     # Extend the recorded added plugins to include the inline plugins (if any)
-    action = operation.actions.only('post_action_data').get(action=ADD_PLUGIN, order=1)
+    action = operation.actions.only("post_action_data").get(action=ADD_PLUGIN, order=1)
     post_data = json.loads(action.post_action_data)
-    post_data['plugins'].extend(get_plugin_data(plugin) for plugin in new_plugins)
+    post_data["plugins"].extend(get_plugin_data(plugin) for plugin in new_plugins)
     action.post_action_data = dump_json(post_data)
-    action.save(update_fields=['post_action_data'])
+    action.save(update_fields=["post_action_data"])
 
 
 def pre_change_plugin(operation, **kwargs):
     from djangocms_history.actions import ADD_PLUGIN, DELETE_PLUGIN
     from djangocms_history.helpers import get_bound_plugins, get_plugin_data
 
-    old_text_plugin = kwargs['old_plugin']
+    old_text_plugin = kwargs["old_plugin"]
     old_plugin_ids = set(old_text_plugin._get_inline_plugin_ids())
 
-    new_text_plugin = kwargs['new_plugin']
+    new_text_plugin = kwargs["new_plugin"]
     new_plugin_ids = set(new_text_plugin._get_inline_plugin_ids())
 
     added_plugin_ids = new_plugin_ids.difference(old_plugin_ids)
@@ -91,20 +93,16 @@ def pre_change_plugin(operation, **kwargs):
     # have already been set on the database when this pre operation
     # is executed.
     old_tree = (
-        old_text_plugin
-        .cmsplugin_set
-        .filter(pk__in=old_plugin_ids)
-        .order_by('position')
-        .values_list('pk', flat=True)
+        old_text_plugin.cmsplugin_set.filter(pk__in=old_plugin_ids)
+        .order_by("position")
+        .values_list("pk", flat=True)
     )
     old_tree = list(old_tree)
 
     new_tree = (
-        new_text_plugin
-        .cmsplugin_set
-        .filter(pk__in=new_plugin_ids)
-        .order_by('position')
-        .values_list('pk', flat=True)
+        new_text_plugin.cmsplugin_set.filter(pk__in=new_plugin_ids)
+        .order_by("position")
+        .values_list("pk", flat=True)
     )
     new_tree = list(new_tree)
 
@@ -115,22 +113,25 @@ def pre_change_plugin(operation, **kwargs):
         order += 1
 
         pre_action_data = {
-            'order': old_tree,
-            'parent_id': old_text_plugin.pk,
+            "order": old_tree,
+            "parent_id": old_text_plugin.pk,
         }
 
-        post_plugin_data = [get_plugin_data(plugin) for plugin in bound_plugins
-                            if plugin.pk in added_plugin_ids]
+        post_plugin_data = [
+            get_plugin_data(plugin)
+            for plugin in bound_plugins
+            if plugin.pk in added_plugin_ids
+        ]
         post_action_data = {
-            'order': new_tree,
-            'parent_id': old_text_plugin.pk,
-            'plugins': post_plugin_data,
+            "order": new_tree,
+            "parent_id": old_text_plugin.pk,
+            "plugins": post_plugin_data,
         }
 
         operation.create_action(
             action=ADD_PLUGIN,
             language=old_text_plugin.language,
-            placeholder=kwargs['placeholder'],
+            placeholder=kwargs["placeholder"],
             pre_data=pre_action_data,
             post_data=post_action_data,
             order=order,
@@ -138,25 +139,29 @@ def pre_change_plugin(operation, **kwargs):
 
     if deleted_plugin_ids:
         order += 1
-        deleted_plugins = [plugin for plugin in bound_plugins if plugin.pk in deleted_plugin_ids]
+        deleted_plugins = [
+            plugin for plugin in bound_plugins if plugin.pk in deleted_plugin_ids
+        ]
         pre_plugin_data = [get_plugin_data(plugin) for plugin in deleted_plugins]
         pre_action_data = {
-            'order': old_tree,
-            'parent_id': old_text_plugin.pk,
-            'plugins': pre_plugin_data,
+            "order": old_tree,
+            "parent_id": old_text_plugin.pk,
+            "plugins": pre_plugin_data,
         }
 
-        post_plugin_data = [get_plugin_data(plugin, only_meta=True) for plugin in deleted_plugins]
+        post_plugin_data = [
+            get_plugin_data(plugin, only_meta=True) for plugin in deleted_plugins
+        ]
         post_action_data = {
-            'order': new_tree,
-            'parent_id': old_text_plugin.pk,
-            'plugins': post_plugin_data,
+            "order": new_tree,
+            "parent_id": old_text_plugin.pk,
+            "plugins": post_plugin_data,
         }
 
         operation.create_action(
             action=DELETE_PLUGIN,
             language=old_text_plugin.language,
-            placeholder=kwargs['placeholder'],
+            placeholder=kwargs["placeholder"],
             pre_data=pre_action_data,
             post_data=post_action_data,
             order=order,
@@ -168,16 +173,18 @@ class TextPlugin(CMSPluginBase):
     name = settings.TEXT_PLUGIN_NAME
     module = settings.TEXT_PLUGIN_MODULE_NAME
     form = TextForm
-    render_template = 'cms/plugins/text.html'
-    change_form_template = 'cms/plugins/text_plugin_change_form.html'
+    render_template = "cms/plugins/text.html"
+    inline_editing_template = "cms/plugins/inline.html"
+    change_form_template = "cms/plugins/text_plugin_change_form.html"
     ckeditor_configuration = settings.TEXT_CKEDITOR_CONFIGURATION
     disable_child_plugins = True
+    fieldsets = ((None, {"fields": ("body",)}),)
 
     # These are executed by the djangocms-history app
     # We use them to inject inline plugin data
     operation_handler_callbacks = {
-        'post_add_plugin': post_add_plugin,
-        'pre_change_plugin': pre_change_plugin,
+        "post_add_plugin": post_add_plugin,
+        "pre_change_plugin": pre_change_plugin,
     }
 
     if CMS_34:
@@ -196,17 +203,22 @@ class TextPlugin(CMSPluginBase):
     def get_translation_export_content(field, plugin_data):
         def _render_plugin_with_content(obj, match):
             from djangocms_translations.utils import get_text_field_child_label
+
             field = get_text_field_child_label(obj.plugin_type)
-            content = getattr(obj, field) if field else ''
+            content = getattr(obj, field) if field else ""
             return plugin_to_tag(obj, content)
 
-        content = _plugin_tags_to_html(plugin_data[field], output_func=_render_plugin_with_content)
+        content = _plugin_tags_to_html(
+            plugin_data[field], output_func=_render_plugin_with_content
+        )
         subplugins_within_this_content = plugin_tags_to_id_list(content)
         return content, subplugins_within_this_content
 
     @staticmethod
     def set_translation_import_content(content, plugin):
-        data = [x.groups() for x in re.finditer(OBJ_ADMIN_WITH_CONTENT_RE_PATTERN, content)]
+        data = [
+            x.groups() for x in re.finditer(OBJ_ADMIN_WITH_CONTENT_RE_PATTERN, content)
+        ]
         data = {int(pk): value for pk, value in data}
 
         return {
@@ -219,23 +231,24 @@ class TextPlugin(CMSPluginBase):
         Returns the Django form Widget to be used for
         the text area
         """
-        cancel_url_name = self.get_admin_url_name('delete_on_cancel')
-        cancel_url = reverse('admin:%s' % cancel_url_name)
+        cancel_url_name = self.get_admin_url_name("delete_on_cancel")
+        cancel_url = reverse("admin:%s" % cancel_url_name)
 
-        render_plugin_url_name = self.get_admin_url_name('render_plugin')
-        render_plugin_url = reverse('admin:%s' % render_plugin_url_name)
+        render_plugin_url_name = self.get_admin_url_name("render_plugin")
+        render_plugin_url = reverse("admin:%s" % render_plugin_url_name)
 
         action_token = self.get_action_token(request, plugin)
 
         # should we delete the text plugin when
         # the user cancels?
         delete_text_on_cancel = (
-            'delete-on-cancel' in request.GET and  # noqa
-            not plugin.get_plugin_instance()[0]
+            "delete-on-cancel" in request.GET
+            and not plugin.get_plugin_instance()[0]  # noqa
         )
 
         widget = TextEditorWidget(
-            installed_plugins=plugins, pk=plugin.pk,
+            installed_plugins=plugins,
+            pk=plugin.pk,
             placeholder=plugin.placeholder,
             plugin_language=plugin.language,
             configuration=self.ckeditor_configuration,
@@ -248,7 +261,9 @@ class TextPlugin(CMSPluginBase):
         return widget
 
     def _get_body_css_classes_from_parent_plugins(
-        self, plugin_instance: CMSPlugin, css_classes: str = '',
+        self,
+        plugin_instance: CMSPlugin,
+        css_classes: str = "",
     ) -> str:
         """
         Recursion that collects CMSPluginBase.child_ckeditor_body_css_class attribute values,
@@ -260,17 +275,22 @@ class TextPlugin(CMSPluginBase):
             for plugin_name, plugin_class in plugin_pool.plugins.items():
                 is_current_parent_found = plugin_name == parent_current.plugin_type
                 if is_current_parent_found:
-                    body_css_class = ''
-                    if getattr(plugin_class, 'child_ckeditor_body_css_class', False):
+                    body_css_class = ""
+                    if getattr(plugin_class, "child_ckeditor_body_css_class", False):
                         body_css_class = plugin_class.child_ckeditor_body_css_class
-                    if getattr(plugin_class, 'get_child_ckeditor_body_css_class', False):
-                        body_css_class = plugin_class.get_child_ckeditor_body_css_class(parent_current)
+                    if getattr(
+                        plugin_class, "get_child_ckeditor_body_css_class", False
+                    ):
+                        body_css_class = plugin_class.get_child_ckeditor_body_css_class(
+                            parent_current
+                        )
 
                     if body_css_class and (body_css_class not in css_classes):
-                        css_classes += ' ' + body_css_class
+                        css_classes += " " + body_css_class
 
             css_classes_collected = self._get_body_css_classes_from_parent_plugins(
-                parent_current, css_classes,
+                parent_current,
+                css_classes,
             )
             if css_classes_collected not in css_classes:
                 css_classes += css_classes_collected
@@ -290,7 +310,7 @@ class TextPlugin(CMSPluginBase):
 
         if instance:
             context = RequestContext(request)
-            context['request'] = request
+            context["request"] = request
             rendered_text = plugin_tags_to_admin_html(
                 text=instance.body,
                 context=context,
@@ -303,26 +323,29 @@ class TextPlugin(CMSPluginBase):
             body = CharField(widget=widget, required=False)
 
             def __init__(self, *args, **kwargs):
-                initial = kwargs.pop('initial', {})
+                initial = kwargs.pop("initial", {})
 
                 if rendered_text:
-                    initial['body'] = rendered_text
+                    initial["body"] = rendered_text
                 super().__init__(*args, initial=initial, **kwargs)
+
         return TextPluginForm
 
     @xframe_options_sameorigin
-    def add_view(self, request, form_url='', extra_context=None):
-        if 'plugin' in request.GET:
+    def add_view(self, request, form_url="", extra_context=None):
+        if "plugin" in request.GET:
             # CMS >= 3.4 compatibility
-            self.cms_plugin_instance = self._get_plugin_or_404(request.GET['plugin'])
+            self.cms_plugin_instance = self._get_plugin_or_404(request.GET["plugin"])
 
-        if getattr(self, 'cms_plugin_instance', None):
+        if getattr(self, "cms_plugin_instance", None):
             # This can happen if the user did not properly cancel the plugin
             # and so a "ghost" plugin instance is left over.
             # The instance is a record that points to the Text plugin
             # but is not a real text plugin instance.
             return super().add_view(
-                request, form_url, extra_context,
+                request,
+                form_url,
+                extra_context,
             )
 
         if not self.has_add_permission(request):
@@ -331,7 +354,7 @@ class TextPlugin(CMSPluginBase):
             # This is NOT the normal workflow because we create a plugin
             # on GET request to the /add/ endpoint and so we bypass
             # django's add_view, thus bypassing permission check.
-            message = gettext('You do not have permission to add a plugin.')
+            message = gettext("You do not have permission to add a plugin.")
             return HttpResponseForbidden(force_str(message))
 
         try:
@@ -341,16 +364,16 @@ class TextPlugin(CMSPluginBase):
             # CMS >= 3.4 compatibility
             _data = self._cms_initial_attributes
             data = {
-                'plugin_language': _data['language'],
-                'placeholder_id': _data['placeholder'],
-                'parent': _data['parent'],
-                'position': _data['position'],
-                'plugin_type': _data['plugin_type'],
-                'plugin_parent': _data['parent'],
+                "plugin_language": _data["language"],
+                "placeholder_id": _data["placeholder"],
+                "parent": _data["parent"],
+                "position": _data["position"],
+                "plugin_type": _data["plugin_type"],
+                "plugin_parent": _data["parent"],
             }
 
         except PermissionDenied:
-            message = gettext('You do not have permission to add a plugin.')
+            message = gettext("You do not have permission to add a plugin.")
             return HttpResponseForbidden(force_str(message))
         except ValidationError as error:
             return HttpResponseBadRequest(error.message)
@@ -359,20 +382,20 @@ class TextPlugin(CMSPluginBase):
         # because we need this record in order to allow the user to add
         # child plugins to the text (image, link, etc..)
         plugin = CMSPlugin.objects.create(
-            language=data['plugin_language'],
-            plugin_type=data['plugin_type'],
-            position=data['position'],
-            placeholder=data['placeholder_id'],
-            parent=data.get('plugin_parent'),
+            language=data["plugin_language"],
+            plugin_type=data["plugin_type"],
+            position=data["position"],
+            placeholder=data["placeholder_id"],
+            parent=data.get("plugin_parent"),
         )
 
         query = request.GET.copy()
-        query['plugin'] = str(plugin.pk)
+        query["plugin"] = str(plugin.pk)
 
-        success_url = admin_reverse('cms_page_add_plugin')
+        success_url = admin_reverse("cms_page_add_plugin")
         # Because we've created the cmsplugin record
         # we need to delete the plugin when a user cancels.
-        success_url += '?delete-on-cancel&' + query.urlencode()
+        success_url += "?delete-on-cancel&" + query.urlencode()
         return HttpResponseRedirect(success_url)
 
     def get_plugin_urls(self):
@@ -381,14 +404,14 @@ class TextPlugin(CMSPluginBase):
             return re_path(regex, func, name=name)
 
         url_patterns = [
-            pattern(r'^render-plugin/$', self.render_plugin),
-            pattern(r'^delete-on-cancel/$', self.delete_on_cancel),
+            pattern(r"^render-plugin/$", self.render_plugin),
+            pattern(r"^delete-on-cancel/$", self.delete_on_cancel),
         ]
         return url_patterns
 
     def get_admin_url_name(self, name):
         plugin_type = self.__class__.__name__.lower()
-        url_name = f'{self.model._meta.app_label}_{plugin_type}_{name}'
+        url_name = f"{self.model._meta.app_label}_{plugin_type}_{name}"
         return url_name
 
     def _get_text_plugin_from_request(self, request, data):
@@ -403,7 +426,7 @@ class TextPlugin(CMSPluginBase):
 
             if text_plugin_id:
                 return self._get_plugin_or_404(text_plugin_id)
-        message = gettext('Unable to process your request. Invalid token.')
+        message = gettext("Unable to process your request. Invalid token.")
         raise ValidationError(message=force_str(message))
 
     @random_comment_exempt
@@ -417,15 +440,17 @@ class TextPlugin(CMSPluginBase):
         form = RenderPluginForm(request.GET, text_plugin=text_plugin)
 
         if not form.is_valid():
-            message = gettext('Unable to process your request.')
+            message = gettext("Unable to process your request.")
             return HttpResponseBadRequest(message)
 
         plugin_class = text_plugin.get_plugin_class_instance()
         # The following is needed for permission checking
         plugin_class.opts = plugin_class.model._meta
 
-        if not (plugin_class.has_change_permission(request, obj=text_plugin) and  # noqa
-                _user_can_change_placeholder(request, text_plugin.placeholder)):
+        if not (
+            plugin_class.has_change_permission(request, obj=text_plugin)
+            and _user_can_change_placeholder(request, text_plugin.placeholder)  # noqa
+        ):
             raise PermissionDenied
         return HttpResponse(form.render_plugin(request))
 
@@ -449,7 +474,7 @@ class TextPlugin(CMSPluginBase):
         form = DeleteOnCancelForm(request.POST, text_plugin=text_plugin)
 
         if not form.is_valid():
-            message = gettext('Unable to process your request.')
+            message = gettext("Unable to process your request.")
             return HttpResponseBadRequest(message)
 
         plugin_class = text_plugin.get_plugin_class_instance()
@@ -459,8 +484,10 @@ class TextPlugin(CMSPluginBase):
         # Check for add permissions because this view is meant
         # only for plugins created through the ckeditor
         # and the ckeditor plugin itself.
-        if not (plugin_class.has_add_permission(request) and  # noqa
-                _user_can_change_placeholder(request, text_plugin.placeholder)):
+        if not (
+            plugin_class.has_add_permission(request)
+            and _user_can_change_placeholder(request, text_plugin.placeholder)  # noqa
+        ):
             raise PermissionDenied
         # Token is validated after checking permissions
         # to avoid non-auth users from triggering validation mechanism.
@@ -478,40 +505,87 @@ class TextPlugin(CMSPluginBase):
         )
         return text_enabled_plugins
 
-    def get_form(self, request, obj=None, **kwargs):
-        plugin = getattr(self, 'cms_plugin_instance', None) or obj
+    def get_plugins(self, obj=None):
+        plugin = getattr(self, "cms_plugin_instance", None) or obj
         get_plugin = plugin_pool.get_plugin
         child_plugin_types = self.get_child_classes(
             slot=plugin.placeholder.slot,
             page=self.page,
         )
         child_plugins = (get_plugin(name) for name in child_plugin_types)
-        plugins = get_toolbar_plugin_struct(
-            child_plugins,
-            plugin.placeholder.slot,
-            self.page,
-        )
+        template = getattr(self.page, "template", None)
+
+        modules = get_placeholder_conf("plugin_modules", plugin.placeholder.slot, template, default={})
+        names = get_placeholder_conf("plugin_labels", plugin.placeholder.slot, template, default={})
+        main_list = []
+
+        # plugin.value points to the class name of the plugin
+        # It's added on registration. TIL.
+        for plugin in child_plugins:
+            main_list.append({'value': plugin.value,
+                              'name': names.get(plugin.value, plugin.name),
+                              'icon': getattr(plugin, "text_icon", None),
+                              'module': modules.get(plugin.value, plugin.module)})
+        return sorted(main_list, key=operator.itemgetter("module"))
+
+    def get_form(self, request, obj=None, **kwargs):
+        plugin = getattr(self, "cms_plugin_instance", None) or obj
+        plugins = self.get_plugins(obj)
         form = self.get_form_class(
             request=request,
             plugins=plugins,
             plugin=plugin,
         )
-        kwargs['form'] = form  # override standard form
+        kwargs["form"] = form  # override standard form
         return super().get_form(request, obj, **kwargs)
 
+    def get_render_template(self, context, instance, placeholder):
+        if hasattr(context["request"], "toolbar") and context["request"].toolbar.edit_mode_active:
+            return self.inline_editing_template
+        else:
+            return self.render_template
+
+    def inline_editing_active(self, request):
+        return (
+            settings.TEXT_INLINE_EDITING
+            and hasattr(request, "toolbar")
+            and request.toolbar.edit_mode_active
+            and request.session.get("inline_editing", True)
+        )
+
     def render(self, context, instance, placeholder):
-        context.update({
-            'body': plugin_tags_to_user_html(
-                instance.body,
-                context,
-            ),
-            'placeholder': placeholder,
-            'object': instance,
-        })
+        if self.inline_editing_active(context["request"]):
+            ckeditor_settings = self.get_editor_widget(
+                context["request"], self.get_plugins(instance), instance
+            ).get_ckeditor_settings(get_language().split("-")[0])
+
+            context.update(
+                {
+                    "body": plugin_tags_to_admin_html(
+                        instance.body,
+                        context,
+                    ),
+                    "placeholder": placeholder,
+                    "object": instance,
+                    "ckeditor_settings": ckeditor_settings,
+                    "ckeditor_settings_id": "ck-cfg-" + str(instance.pk),
+                }
+            )
+        else:
+            context.update(
+                {
+                    "body": plugin_tags_to_user_html(
+                        instance.body,
+                        context,
+                    ),
+                    "placeholder": placeholder,
+                    "object": instance,
+                }
+            )
         return context
 
     def save_model(self, request, obj, form, change):
-        if getattr(self, 'cms_plugin_instance', None):
+        if getattr(self, "cms_plugin_instance", None):
             # Because the plugin was created by manually
             # creating the CMSPlugin record, it's important
             # to assign all the values from the CMSPlugin record
@@ -540,11 +614,8 @@ class TextPlugin(CMSPluginBase):
 
     def _get_plugin_or_404(self, pk):
         plugin_type = self.__class__.__name__
-        plugins = (
-            CMSPlugin
-            .objects
-            .select_related('placeholder', 'parent')
-            .filter(plugin_type=plugin_type)
+        plugins = CMSPlugin.objects.select_related("placeholder", "parent").filter(
+            plugin_type=plugin_type
         )
 
         field = self.model._meta.pk
@@ -552,7 +623,7 @@ class TextPlugin(CMSPluginBase):
         try:
             object_id = field.to_python(unquote(pk))
         except (ValidationError, ValueError):
-            raise Http404('Invalid plugin id')
+            raise Http404("Invalid plugin id")
         return get_object_or_404(plugins, pk=object_id)
 
 
