@@ -71,7 +71,7 @@ var CKBUILDER_CONFIG = {
 		'entities' : 1,
 		'filebrowser' : 1,
 		'find' : 1,
-		'flash' : 1,
+//		'flash' : 1,
 		'floatingspace' : 1,
 		'font' : 1,
 		'format' : 1,
@@ -187,4 +187,4 @@ var CKBUILDER_CONFIG = {
 		'zh' : 1,
 		'zh-cn' : 1
 	}
-};
\ No newline at end of file
+};
