@@ -1,5 +1,6 @@
 import json
 from copy import deepcopy
+from itertools import groupby
 
 from django import forms
 from django.conf import settings
@@ -7,7 +8,7 @@ from django.core.serializers.json import DjangoJSONEncoder
 from django.template.loader import render_to_string
 from django.templatetags.static import static
 from django.utils.safestring import mark_safe
-from django.utils.translation.trans_real import get_language
+from django.utils.translation.trans_real import get_language, gettext
 
 from cms.utils.urlutils import static_with_version
 
@@ -15,7 +16,7 @@ from . import settings as text_settings
 
 
 # this path is changed automatically whenever you run `gulp bundle`
-PATH_TO_JS = 'djangocms_text_ckeditor/js/dist/bundle-fb7663a06e.cms.ckeditor.min.js'
+PATH_TO_JS = 'djangocms_text_ckeditor/js/dist/bundle-1c5205cc04.cms.ckeditor.min.js'
 
 
 class TextEditorWidget(forms.Textarea):
@@ -41,21 +42,21 @@ class TextEditorWidget(forms.Textarea):
             'data-ckeditor-basepath': text_settings.TEXT_CKEDITOR_BASE_PATH,
         })
         super().__init__(attrs)
-        self.installed_plugins = installed_plugins
-        self.pk = pk
-        self.placeholder = placeholder
+        self.installed_plugins = installed_plugins  # general
+        self.pk = pk  # specific
+        self.placeholder = placeholder  # specific
         self.plugin_language = plugin_language
         if configuration and getattr(settings, configuration, False):
             conf = deepcopy(text_settings.CKEDITOR_SETTINGS)
             conf.update(getattr(settings, configuration))
-            self.configuration = conf
+            self.configuration = conf  # specific
         else:
-            self.configuration = text_settings.CKEDITOR_SETTINGS
-        self.cancel_url = cancel_url
-        self.render_plugin_url = render_plugin_url
-        self.action_token = action_token
-        self.delete_on_cancel = delete_on_cancel
-        self.body_css_classes = body_css_classes
+            self.configuration = text_settings.CKEDITOR_SETTINGS  # general
+        self.cancel_url = cancel_url  # specific
+        self.render_plugin_url = render_plugin_url  # specific
+        self.action_token = action_token  # specific
+        self.delete_on_cancel = delete_on_cancel  # specific
+        self.body_css_classes = body_css_classes  # specific
 
     @property
     def media(self):
@@ -72,10 +73,7 @@ class TextEditorWidget(forms.Textarea):
     def render_textarea(self, name, value, attrs=None, renderer=None):
         return super().render(name, value, attrs, renderer)
 
-    def render_additions(self, name, value, attrs=None, renderer=None):
-        # id attribute is always present when rendering a widget
-        ckeditor_selector = attrs['id']
-        language = get_language().split('-')[0]
+    def get_ckeditor_settings(self, language):
         configuration = deepcopy(self.configuration)
         # We are in a plugin -> we use toolbar_CMS or a custom defined toolbar
         if self.placeholder:
@@ -90,15 +88,49 @@ class TextEditorWidget(forms.Textarea):
             configuration['toolbar'] = configuration.get('toolbar', 'HTMLField')
 
         configuration['bodyClass'] = self.body_css_classes
-
         config = json.dumps(configuration, cls=DjangoJSONEncoder)
+
+        # Group plugins by module
+        if self.installed_plugins:
+            plugins = groupby(sorted(self.installed_plugins, key=lambda x: x.get("module")), key=lambda x: x.get("module"))
+            plugins = [{'group': group, 'items':
+                [{'title': item.get('name'), 'type': item.get('value')} for item in items]} for group, items in plugins]
+        else:
+            plugins = []
+
+        return {
+            'language': language,
+            'installed_plugins': self.installed_plugins,
+            'static_url': settings.STATIC_URL + 'djangocms_text_ckeditor',
+            'plugin_id': self.pk,
+            'plugin_language': self.plugin_language,
+            'placeholder_id': self.placeholder.pk if self.placeholder else None,
+            'render_plugin_url': self.render_plugin_url or '',
+            'add_plugin_url': self.placeholder.get_add_url() or '' if self.placeholder else '',
+            'clancel_plugin_url': self.cancel_url or '',
+            'delete_on_cancel': self.delete_on_cancel or False,
+            'action_token': self.action_token or '',
+            'lang': {
+                'toolbar': gettext('CMS Plugins'),
+                'add': gettext('Add CMS Plugin'),
+                'edit': gettext('Edit CMS Plugin'),
+                'aria': gettext('CMS Plugins'),
+            },
+            'plugins': plugins,
+            'options': json.loads(config.replace('{{ language }}', language)),
+        }
+
+    def render_additions(self, name, value, attrs=None, renderer=None):
+        # id attribute is always present when rendering a widget
+        ckeditor_selector = attrs['id']
+        language = get_language().split('-')[0]
+
         context = {
             'ckeditor_class': self.ckeditor_class,
             'ckeditor_selector': ckeditor_selector,
             'ckeditor_function': ckeditor_selector.replace('-', '_'),
             'name': name,
             'language': language,
-            'settings': config.replace('{{ language }}', language),
             'STATIC_URL': settings.STATIC_URL,
             'CKEDITOR_BASEPATH': text_settings.TEXT_CKEDITOR_BASE_PATH,
             'installed_plugins': self.installed_plugins,
@@ -107,6 +139,8 @@ class TextEditorWidget(forms.Textarea):
             'placeholder': self.placeholder,
             'widget': self,
             'renderer': renderer,
+            'ckeditor_settings': self.get_ckeditor_settings(language),
+            'ckeditor_settings_id': 'ck-cfg-' + (str(self.pk) if self.pk else ckeditor_selector),
         }
         return mark_safe(render_to_string('cms/plugins/widgets/ckeditor.html', context))
 
